# Ramses Transliteration Corpus, V. 2019-09-01

![](cc-by-nc-sa.png) *Université de Liège/Projet Ramsès & Conservatoire National des Arts et métiers, laboratoire Cédric.*

- The data in this project has been created by the team of the [Ramses Project](https://ramses.ulg.ac.be);
- The software has been written and experimented by [Serge Rosmorduc](http://cedric.cnam.fr/lab/en/author/Rosmorduc/)

This project contains the Ramses transliteration corpus and software, which has been created to train Machine Learning Algorithms on the task of transliterating Late Egyptian.

The corpus is released using the [CC-BY-NC-SA Creative Common License](https://creativecommons.org/licenses/by-nc-sa/4.0/). Please acknowledge its use as "the Ramses transliteration corpus V. 2019-09-01, University of Liege/Projet Ramses".

## License

[*Ramses Automated Transliterator*](https://gitlab.cnam.fr/gitlab/rosmorse/ramses-trl) by [**Université de Liège/Projet Ramsès**](http://ramses.ulg.ac.be/) & [**Serge Rosmorduc**](http://cedric.cnam.fr/lab/en/author/Rosmorduc/)
is licensed under `CC BY-NC-SA` 4.0. To view a copy of this license, visit https://creativecommons.org/licenses/by-nc-sa/4.0

![Creative Common License](https://chooser-beta.creativecommons.org/img/cc.7a093a7d.svg) ![Credits must be given to the creators of the project](https://chooser-beta.creativecommons.org/img/by.f6aa22c4.svg) ![Only noncommercial use of this work is permitted](https://chooser-beta.creativecommons.org/img/nc.8c3b7ea6.svg) ![Adaptations must be shared under the same terms](https://chooser-beta.creativecommons.org/img/sa.67ddd908.svg)

Basically: 

- respect our authorship;
- no commercial reuse;
- sharing and modifications under the same conditions.

## Running with Docker

In strongly suggest using docker if you simply want to play a little with the system, especially with the Java UI.

### Command-line application

I provide an "easy" (or, let's say, somehow reliable) way to run the software, if you have installed [Docker](https://www.docker.com/)

1. build the docker container :

   ~~~
   docker build -t translit:1.0 .
   ~~~

2. run the docker container, with a list of Gardiner codes as arguments

   ~~~
   docker run translit:1.0  "M17 G43 D21 D36 N5 G17 X1 G1 Q3 X1 N1"
   ~~~

   You will get plenty of technical messages, with the processed text at the end :

   ~~~
   2020-11-30 13:50:55.213909: W tensorflow/stream_executor/platform/default/dso_loader.cc:60] Could not load dynamic ...
   (and sundry other messages)
   2020-11-30 13:50:58.401285: I tensorflow/core/platform/profile_utils/cpu_utils.cc:112] CPU Frequency: 2400000000 Hz
   iw ra m tA p.t 
   ~~~

   Here, the result is `iw ra m p.t`.

### The Java GUI

I provide a relatively simple Java GUI. It uses a small python server

- first, build (once!) a docker image for the server :

   ~~~
   docker build -f Dockerfile-serv -t trls:1.0 .
   ~~~

- run the server :

    ~~~
    docker run -p 8080:8080 trls:1.0
    ~~~

- go to the `java_ui` folder and run the software with:

  ~~~
  ./gradlew run
  ~~~

- A window will open. The top part is a `JSesh` text area. You can type JSesh text or copy/paste it there. Then, click on the button in
  the bottom. The transliteration should appear below. If nothing happens, the simplest guest is that you haven't started the server,
  or you have stopped it.

## Content of the archive

This archive contains:

- the data itself (see below) in the folder `data`;
- the code used in our article in the folder `python`;
- some data extracted from the corpus in the folder `results`.

## Corpus

The corpora are lists of sentences, organised on a line basis. One line is a sentence.

In **src** (source) files, you will find lists of glyph codes, like this:
~~~~~
N11 Z2 O1 D21 X1 N5 LACUNA 
N5 Z1 Q3 N35
~~~~~

Each **src** file has a corresponding **tgt** (target) file, which corresponds to the transliteration of the source. The target files are space separated lists of transliteration symbols:
~~~~
A b d _ 3 _ p r . t _ r a _ L A C U N A _ 
h r w _ p n _ 
~~~~
Underscores (_) are used to indicate word separations.

The following sources files are available:

- src-train.txt: training corpus
- src-val.txt: validation corpus
- src-test.txt: test corpus
- src-ctest.txt: test corpus with only hieratic texts
- src-htest.txt: test corpus with only hieroglyphic texts
- src-sailor.txt: shipwrecked sailor (Middle Egyptian text) for testing purposes.
- src-sep-train.txt, src-sep-val.txt: source corpora with word separation in the input (see below)

The target (gold) corpora have the same names as the source corpora, except that `src` is replaced with `tgt`. So `src-train.txt` corresponds to `tgt-train.txt`. We provide source files both with and without word separation. The source corpora with included separation have the same gold target corpora as the plain source corpora. So the tgt file which corresponds to `src-sep-cval.txt` is `tgt-cval.txt`.



### Corpus limitations

The corpus was created in a mechanical way, from the annotations in the Ramses corpus. This has a number of consequences, which are detailed in the article:

- transliterations are standardised according to what is grammatically expected. The infinitive of "jrj" will be transliterated as "jrj.t", regardless of the actual spelling of the word.
- the linear organisation of the Ramses database doesn't allow a proper representation of honorific transposition. Hence, except in some proper names, the annotated data lists the words in the order in which they graphically occur. It will say *jmn mrj* and not *mrj jmn*.
- there are a few problems in the output, which should be fixed in later versions of the corpus. Some cases of partial lacuna are not properly marked in transliteration, for instance.

## Software

The folder "python" contains the python files needed to train and process the corpus.
Note that it might be easier to run using the docker files.

### Preparing the environment

To use this software, you need an installation of python (version 3.5.2, along with the following libraries:

- keras (2.1.5)
- numpy (1.14.1)
- matplotlib (1.5.2)

The version numbers we give are simply those which are currently installed on our machine. You can probably work with other versions. The only really important information is that you must use Python 3.

### Running the transliterator

You first need a trained network. The easiest way to go is to download the pre-trained network we publish at: http://files.qenherkhopeshef.org/ramses/network.h5

Note that, due to the large size of the network file, we may move it later to another server. In this case, we will update the present documentation.

Download it, and copy it in the folder called "network".

To run the transliterator with the downloaded network, type:

Then go to the `python` folder and type:
~~~~~
python3 translit.py --net ../network/network.h5 --mdc "M17 G43 D21 D36 C1 G17 Q3 X1 N1"
~~~~~
(if running Windows, replace "/" by "\")

You might get some warning (due to variant versions of python), and then the result:

~~~
python3 translit.py --net ../network/network.h5 --mdc "M17 G43 D21 D36 C1 G17 Q3 X1 N1"
__init__.py:36: FutureWarning: Conversion of the second argument of issubdtype from `float` to `np.floating` is deprecated. In future, it will be treated as `np.float64 == np.dtype(float).type`.
  from ._conv import register_converters as _register_converters
Using TensorFlow backend.
_bootstrap.py:222: RuntimeWarning: compiletime version 3.6 of module 'tensorflow.python.framework.fast_tensor_util' does not match runtime version 3.5
  return f(*args, **kwds)
2020-08-24 17:37:14.245819: I tensorflow/core/platform/cpu_feature_guard.cc:140] Your CPU supports instructions that this TensorFlow binary was not compiled to use: AVX2 FMA
iw ra m p.t 
~~~
Here, the result is `iw ra m p.t`.

The provided network characteristics are originally:

- bidi architecture
- using segmentation information in *training*
- size of hidden layers is 500
- epoch 180
- accuracy 0.9254
- loss 0.1530

### Possible arguments

The command `translit.py`accepts the following arguments (default values are indicated in *italic*)

* **-h, --help**: show an help message and exit
* **--net NET**:  saved network to use **mandatory**
* **--arch ARCH**: network architecture: plain, bidi, seg or *bidiseg*
* **--hidden HIDDEN**: size of hidden layer (*500*)
* **--beam BEAM**: beam width (*1*)
* **--corpus CORPUS**: corpus to transliterate: *val*, test, htest, ctest
* **--mdc MDC**: sentence to transliterate
* **--compact** if not set, all characters will be separated by spaces (useful for batch and test)
* **--attention ATTENTION_PICT** filename for attention map
* **--display_attention**  interactive display of the attention map

The `--arch` and `--hidden` must correspond to the values used when creating the network (for the provided system, those values are respectively `bidiseg` and `500`. `bidi` would work too) 

The text to transliterate might be a specified corpus, defined by `--corpus`, or a list of manuel de codage codes, introduced by `--mdc`. The glyphs codes **must** at this point be Gardiner code, no phonetic mnemonics. The system is able to deal with some positioning information like ':' or '*', but it's safer if you use a space separated list of codes.

The `--beam` corresponds to the number of hypothesis the system will explore at the same time. The default, with a beam width of 1, will always use the transliteration which is seen as the most likely at each step. *This doesn't always lead to the best global solution !* It's possible that, two step later, a solution which was seemed as less likely will produce a result which is overall better. With a beam width of *n*, the system will keep in memory the *n* best solutions it currently knows (and discard the other solutions). This is still false --- the only solution would be to keep *all* the solutions, but their number is way too large. A beam width of *5* is a reasonable trade-off.

The default *beam width* is 1, to speed up testing. Feel free to use larger widths, however.

### Training a transliterator

It's possible to use the software for training your own version of the transliterator, especially if you have your own corpus. To do this, you need an installation of python, a GPU, and time (the smallest GPU we used was an NVidia 1060 with 6GB of RAM, which is a relatively modest one; it took us approximatively one day to train). 

To train a new network, use the following command line:
~~~
python3 train_bidi_segment_h500.py
~~~

If you want to change the network's parameters, copy and edit the file `train_bidi_segment_h500.py`. It's relatively small and I hope reasonably commented (although sometimes in french, I haven't been too careful there).

### Computing the Average Levenshtein Distance


The Average Levenshtein Distance for a corpus can be computed using `average_levenshtein_distance.py`.

The systems evaluates the results against the validation corpus. For other corpora, use the argument --corpus (with values val, test, htest or ctest)

Example: first, we run the network on the validation corpus.
~~~~
python3 translit.py --net ../network/network.h5  > result.txt
~~~~

The result is kept in the file `result.txt`. We compare it with the "gold" corpus:
~~~~
python3 average_levenshtein_distance.py result.txt
~~~~

The software will output:

- first, **for each sentence**:

  - a sentence number
  - the length of the expected output
  - the length of the actual output
  - the Levenshtein distance between the expected output and the actual output
  
- then a table giving the total results for each *length* of sentence. Its columns are:
    - l: size of the gold output
    - n: number of sentences of size l
    - s: sum of levenshtein distances between gold and actual output
    - m: s/(n*l) estimation of levenshtein distance by sign for length l
    - N: total number of *signs* seen in output
    - S: total error from now
    - M: S / N

Thus, the bottom-right of the table gives the *average levenshtein distance* for the whole corpus.

## Resulting data
The `results` folder contains a number of outputs built with the system.

### sign_values.gly

A readable JSesh file listing all signs values extracted from the system (see the articles for explanation).

### values.csv

The source for all "A" values (see the article for details), as a semicolumn-separated CSV file. The columns are:

- number of occurrences
- sign code
- value

A sign which has multiple values will appear on multiple lines.

### secondary.csv
The source for all "B" values (see the article for details), as a semicolumn-separated CSV file. The columns are:

- number of occurrences
- sign code
- value

### sailor-trl-naufrage-processed.gly
A JSesh file, with the text of the shipwrecked sailor, its transliteration by our system 
(with a comparison with the "gold" transliteration).


## Bibliography

- Serge Rosmorduc (2020), “Automated Transliteration of Late Egyptian Using Neural Networks: An Experiment in “Deep Learning””, *Lingua Aegyptia* 28 (233–257). (DOI: https://doi.org/10.37011/lingaeg.28.07)
