from sys import argv, stderr, exit
from translit_lib.levenshtein import LevenshteinDistance
import argparse


def compare(path, corpus='val'):
    fic1 = '../data/tgt-'+corpus+'.txt'
    fic2 = path
    distance = LevenshteinDistance()
    distance.fullDist(fic1,fic2)
    res = distance.meanDist(fic1, fic2)
    print('''please note that the distance can be larger than l, 
    because l is only the length of the expected output, 
    whereas the actual output might be larger (especially if it starts repeating itself)
    Legend : 
      - l : size of the gold output
      - n : number of sentences of size l
      - s : sum of levenshtein distances between gold and actual output
      - m : s/(n*l) estimation of levenshtein distance by sign for length l
      - N : total number of *signs* seen in output
      - S : total error from now
      - M : S / N
    ''')
    print("l\tn\ts\tm\tN\tS\tM")
    for key, value in sorted(res.items()):        
        print("{}\t{}\t{:7.3f}\t{:7.3f}\t{}\t{:7.3f}\t{:7.3f}".format(key, *value))

def _startSoft():
    usage= '''
    Computes Average Levenshtein Distance between a computed result and a gold corpus.
    '''
    parser = argparse.ArgumentParser(description=usage)
    parser.add_argument("path", help='path of the computed transliteration file to evaluate' )
    parser.add_argument("--corpus",  dest="corpus", help="corpus to evaluate against : val, test, htest, ctest", default="val") 
    if len(argv)==1:
        parser.print_help(stderr)
        exit(1)
    args = parser.parse_args()
    compare(args.path, args.corpus)


if __name__ == "__main__":
    _startSoft()
    



