"""
Simple server, with fewer bells and whistles than "translit.py"
No need here for a full REST API...

Possible requests :

http://localhost:8080/stop
http://localhost:8080/translit
http://localhost:8080/attention

- Both on GET and POST
- for the last two, mdc="..." will contain the mdc text to analyse.
"""

from http.server import BaseHTTPRequestHandler, HTTPServer
import urllib.parse as urlparse
from http.client import parse_headers
import logging
import os

from matplotlib.pyplot import winter

from translit_lib.vocabulary import Vocabulary
from translit_lib.corpus_utils import BatchCorpus, create_and_train, computeRepresentation
from translit_lib.encoder_decoder import Encoder, Decoder, BidiEncoder, Transliterator
from translit_lib.beam import Beam

# Settings for the network
beam = 5
hidden = 500
network = "network/network.h5"


def load_network():
    pass
    vocabulary = Vocabulary()
    vocabulary.loadDefault()
    # Boolean indicator needed to state we want to keep attention information.
    # attention_output=(attention_pict != None) or display_attention    
    encoder = BidiEncoder(input_size=vocabulary.size(), hidden=hidden)
    encoder_model = encoder.load_model_from_file(network)
    decoder = Decoder(hidden=hidden)
    decoder_model = decoder.load_model_from_file(network, attention_output=True)
    transliterator = Transliterator(vocabulary, encoder_model, decoder_model, attention_output=True)
    return transliterator

transliterator = load_network()

def transliterate(mdc):
    logging.info("transliterating {}".format(mdc))
    trl = transliterator.transliterate(mdc, beam_width=beam)
    logging.info("result {}".format(trl))
    return trl

class TranslitServlet(BaseHTTPRequestHandler):
    def _prepare_reponse(self):
        self.send_response(200)
        self.send_header('Content-type', 'text/plain')
        self.end_headers()

    def _prepare_picture(self):
        self.send_response(200)
        self.send_header('Content-type', 'image/png')
        self.end_headers()

    def do_GET(self):
        logging.info("GET")
        parsed_url = urlparse.urlparse(self.path)
        parameters = urlparse.parse_qs(parsed_url.query)
        path = parsed_url.path[1:]
        self.process_request(path, parameters)
        

    def do_POST(self):
        logging.info("POST")
        content_length = int(self.headers.get('content-length'))
        post_data = self.rfile.read(content_length).decode('utf-8')
        parameters = urlparse.parse_qs(post_data)        
        self.process_request(self.path[1:], parameters)

        
    def process_request(self,command,parameters):
        logging.info("receiving command {}\n".format(command))
        if command == "stop":
            self.stop()
        elif command == "transliterate":
            self.transliterate(parameters["mdc"][0])
        elif command == "attention":
            self.attention(parameters["mdc"][0])
        else:
            self._prepare_reponse()        
            self.wfile.write("UNKNOWN COMMAND {}".format(command).encode('utf-8'))            
        pass
        logging.info("REQUEST")
        
    def attention(self, mdc):
        """
        Returns the attention for a given string...
        """
        output = "tmp.png"
        logging.info("computing attention for {}".format(mdc))
        transliterator.transliterate(mdc)
        transliterator.plot_attention(picture=output)
        logging.info("computing attention for {}".format(mdc))
        logging.info(str(os.stat(output)))
        self._prepare_picture()
        with open(output, "rb") as img :            
            self.wfile.write(img.read())         

    def transliterate(self, mdc):
        result = transliterate(mdc)
        self._prepare_reponse()        
        self.wfile.write("{}".format(result).encode('utf-8'))            

    def stop(self):
        self._prepare_reponse()
        self.wfile.write("BYE".encode('utf-8'))            
        exit(0)
    

def run(server_class=HTTPServer, handler_class=TranslitServlet, port=8080):
    logging.basicConfig(level=logging.INFO)
    server_address = ('', port)
    httpd = server_class(server_address, handler_class)
    logging.info('Starting httpd...\n')
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass
    httpd.server_close()
    logging.info('Stopping httpd...\n')

if __name__ == '__main__':   
    run()
