"""
Trains a network with a bidirectional encoder, segmentation output, and hidden layers of width 500.
"""
import keras
from translit_lib.vocabulary import Vocabulary
from translit_lib.corpus_utils import BatchCorpus, create_and_train, CorpusSegmentation
from translit_lib.encoder_decoder import EncoderDecoderBidiSegmentation

vocabulary = Vocabulary()
vocabulary.loadDefault()

src = "../data/src-sep-train.txt"
tgt = "../data/tgt-train.txt"

src_val = "../data/src-sep-val.txt"
tgt_val = "../data/tgt-val.txt"

# The corpora, including the segmentation task.

corpus = CorpusSegmentation(vocabulary=vocabulary, src=src, tgt=tgt, batch_size=32)

corpus_val = CorpusSegmentation(vocabulary=vocabulary, 
                         src=src_val, tgt=tgt_val, batch_size=32)

# A function to create the model (needed for "create_and_train" below)
def model_generator():
    encoder_decoder_seg = EncoderDecoderBidiSegmentation(hidden=500, 
                            encoder_input_size=vocabulary.size(), decoder_input_size= 256)
    model =  encoder_decoder_seg.build_model()
    model.summary()
    return model

# Create and train the model.
model = create_and_train("../network/bidi-segment-user-500h-e{epoch:03d}-a{val_softmax_acc:.4f}-loss{val_softmax_loss:.4f}.h5",
                        model_generator, 
                        corpus, period=20, num_epochs=200,
                        validation_data=corpus_val.no_batch()
)

